<?php
defined( 'ABSPATH' ) || exit; // Exit if accessed directly.

if ( ! class_exists( 'Safelayout_Preloader_Front' ) ) {

	class Safelayout_Preloader_Front {
		protected $options = null;
		protected $active_loader = false;
		protected $main_code_added = false;
		protected $allowed_tags = array(
			'noscript'		=> [],
			'span'			=> [ 'style'		=> 1, ],
			'style'			=> [ 'id'			=> 1, ],
			'defs'			=> [ 'id'			=> 1, ],
			'mask'			=> [ 'id'			=> 1, ],
			'g'				=> [ 'filter'		=> 1, ],
			'fegaussianblur'=> [ 'stddeviation'	=> 1, ],
			'div'			=> [
				'class'			=> 1,
				'id'			=> 1,
				'style'			=> 1,
			],
			'img'			=> [
				'class'			=> 1,
				'data-*'		=> 1,
				'id'			=> 1,
				'style'			=> 1,
				'alt'			=> 1,
				'src'			=> 1,
			],
			'svg'			=> [
				'class'			=> 1,
				'viewbox'		=> 1,
			],
			'lineargradient'=> [
				'id'			=> 1,
				'x1'			=> 1,
				'y1'			=> 1,
				'x2'			=> 1,
				'y2'			=> 1,
			],
			'stop'			=> [
				'stop-color'	=> 1,
				'stop-opacity'	=> 1,
				'offset'		=> 1,
			],
			'path'			=> [
				'class'			=> 1,
				'stroke'		=> 1,
				'fill'			=> 1,
				'mask'			=> 1,
				'id'			=> 1,
				'd'				=> 1,
			],
			'circle'		=> [
				'class'			=> 1,
				'stroke'		=> 1,
				'mask'			=> 1,
				'cx'			=> 1,
				'cy'			=> 1,
				'r'				=> 1,
			],
			'rect'			=> [
				'x'				=> 1,
				'y'				=> 1,
				'width'			=> 1,
				'height'		=> 1,
				'fill'			=> 1,
			],
			'filter'		=> [
				'x'				=> 1,
				'y'				=> 1,
				'width'			=> 1,
				'height'		=> 1,
				'id'			=> 1,
			],
		); 

		public function __construct() {
			add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_scripts' ), 1 );
			add_action( 'wp_body_open', array( $this, 'body_open' ), 1 );
			add_action( 'wp_footer', array( $this, 'set_footer' ) );

     
			add_filter( 'safe_style_css', function( $styles ) {
				$styles[] = 'top';
				$styles[] = 'left';
				$styles[] = 'opacity';
				$styles[] = 'position';
				$styles[] = 'animation-delay';
				$styles[] = '-webkit-animation-delay';
				return $styles;
			} );
		}

		// Add css style and js script
		public function enqueue_scripts() {
			$this->main_code_added = false;
			$this->check_active_loader();
			if ( $this->active_loader ) {
				$options = $this->options;
				if ( ! $options['inline_script'] ) {
					wp_enqueue_script(
						'safelayout-cute-preloader-script',
						SAFELAYOUT_PRELOADER_URL . 'assets/js/safelayout-cute-preloader.min.js',
						array(),
						SAFELAYOUT_PRELOADER_VERSION,
						true
					);
					$temp_obj = array(
						'showingLoader'		=> true,
						'pageLoaded'		=> false,
						'minShowTime'		=> esc_html( $options['minimum_time'] ) * 1000,
						'maxShowTime'		=> esc_html( $options['maximum_time'] ) * 1000,
						'showCloseButton'	=> esc_html( $options['close_button'] ) * 1000,
					);
					wp_localize_script( 'safelayout-cute-preloader-script', 'slplPreLoader', $temp_obj );
				}
				if ( ! $options['inline_style'] ) {
					$group1 = array( 'window', 'infinite', 'crawl', 'turn', 'turn1', '3d-bar', 'wheel', );
					$group2 = array( '3d-plate', 'blade-horizontal', 'blade-horizontal1', 'blade-vertical', 'blade-vertical1', 'balloons', );
					$group3 = array( 'gear', 'trail', 'planet', 'cube', 'cube1', 'cycle', 'stream', 'tube', 'leap', );
					$group4 = array( 'queue', 'jump', 'octagon', 'octagon1', 'grid', );

					$icon_group = 5;
					if ( in_array( $options['icon'], $group1 ) ) {
						$icon_group = 1;
					} else if ( in_array( $options['icon'], $group2 ) ) {
						$icon_group = 2;
					} else if ( in_array( $options['icon'], $group3 ) ) {
						$icon_group = 3;
					} else if ( in_array( $options['icon'], $group4 ) ) {
						$icon_group = 4;
					}

					wp_enqueue_style(
						'safelayout-cute-preloader-style',
						SAFELAYOUT_PRELOADER_URL . 'assets/css/safelayout-cute-preloader-front-icon-group' . $icon_group . '.min.css',
						array(),
						SAFELAYOUT_PRELOADER_VERSION
					);
				} else {
					require_once SAFELAYOUT_PRELOADER_PATH . 'inc/safelayout-preloader-set-inline-style.php';
					safelayout_preloader_set_inline_style( $options );
				}
				echo wp_kses( stripslashes( $options['code_CSS'] ), $this->allowed_tags );
			}
		}

		// Add html after body tag
		public function body_open() {
			if ( ! $this->main_code_added && $this->active_loader ) {
				$this->main_code_added = true;
				echo wp_kses( stripslashes( $this->options['code_HTML'] ), $this->allowed_tags );
			}
		}

		// Add inline script and add html if no body_open
		public function set_footer() {
			if ( $this->active_loader ) {
				if ( ! $this->main_code_added ) {
					$this->main_code_added = true;
					echo wp_kses( stripslashes( $this->options['code_HTML'] ), $this->allowed_tags );
				}

				$options = $this->options;
				$temp0 = array( 'progress-opacity-to-right', 'progress-opacity-to-left', 'progress-opacity-to-top', 'progress-opacity-to-bottom', );
				if ( $options['bar_shape'] != 'No' ||
					$options['counter'] === 'enable' ||
					( trim( $options['brand_url'] ) !== '' && in_array( $options['brand_anim'], $temp0 ) ) ) {
						$this->set_progress_bar_script();
				}
				if ( $options['inline_script'] ) {
					$this->set_inline_script();
				}
				if ( trim( $options['brand_url'] ) !== '' ) {
					$temp1 = array( 'wrest-X', 'wrest-Y', 'roll', 'pipe', 'swirl', 'sheet', );
					if ( in_array( $options['brand_anim'], $temp1 ) ) {
						echo '<script id = "safelayout-cute-preloader-brand-anim-synchro" type = "text/javascript">' .
							 "\n\tdocument.addEventListener('DOMContentLoaded', function() {" .
							 "\n\t\tvar childs = document.getElementById('sl-pl-brand-parent').children;" .
							 "\n\t\tvar name = 'sl-pl-brand-" . esc_html( $options['brand_anim'] ) . "';" .
							 "\n\t\tfor ( var i = 0 ; i < childs.length ; i++ ) {\n\t\t\tif ( childs[i].classList ) {\n\t\t\t\tchilds[i].classList.add( name );" .
							 "\n\t\t\t} else {\n\t\t\t\tchilds[i].className += ' ' + name;\n\t\t\t}\n\t\t}" . "\n\t});\n</script>";
					}
				}
			}
			$this->options = null;
		}

		// Add progress bar script
		public function set_progress_bar_script() {
			$options = $this->options;
			$brand = '';
			$style = 'width';
			$temp0 = array( 'progress-opacity-to-right', 'progress-opacity-to-left', 'progress-opacity-to-top', 'progress-opacity-to-bottom', );
			if ( trim( $options['brand_url'] ) !== '' && in_array( $options['brand_anim'], $temp0 ) ) {
				$brand = $options['brand_anim'];
				if ( strpos( $brand, 'top' ) || strpos( $brand, 'bottom' ) ) {
					$style = 'height';
				}
			}
			?>
			<script id = "safelayout-cute-preloader-progress-bar-script-js" type = "text/javascript">
				function slplSetTimer( time, max ) {
					return setInterval( function () {
							var slplTemp = Math.min( Math.floor( slplPercent ), 100 );
							if ( slplProgress ) {
								slplProgress.style.width = slplTemp + "%";
							}
							if ( slplBrand ) {
								slplBrand.style.<?php echo esc_html( $style ); ?> = slplTemp + "%";
							}
							if ( slplCounter ) {
								slplCounter.innerHTML = slplTemp + "<?php echo '<span>' . esc_html( $options['counter_text'] ) . '</span>'; ?>";
							}
							if ( slplPercent >= max ) {
								clearInterval( slplInterval );
								return;
							}
							slplPercent += slplStep;
						}, time );
				}
				function slplResourceComplete() {
					if ( slplPercent < 100 ) {
						clearInterval( slplInterval );
						slplStep = ( slplPercent < 40 ) ? 3 : ( slplPercent < 70 ) ? 2.5 :( slplPercent < 80 ) ? 2 : 1;
						slplInterval = slplSetTimer( 5, 100 );
					}
					return slplPercent;
				}
				document.addEventListener( 'readystatechange', function() {
					if ( document.readyState === 'complete' ) {
						slplResourceComplete();
					}
				});
				var slplImgs = document.images,
					slplVids = document.getElementsByTagName('video'),
					slplProgress = document.getElementById('sl-pl-progress'),
					slplCounter = document.getElementById('sl-pl-counter'),
					slplBrand = <?php echo ( ( $brand != '' ) ? "document.getElementById('sl-pl-" . esc_html( $brand ) . "'),\n" : "null,\n" ); ?>
					slplMax = Math.floor(Math.random() * 10 + 80),
					slplResource = 0,
					slplPercent = 0,
					slplStep = 1,
					slplInterval;
				for ( var i = 0, len = slplImgs.length; i < len; i++ ) {
					if ( slplImgs[i].loading != 'lazy' && ! slplImgs[i].complete && slplImgs[i].src ) {
						slplResource++;
					}
				}
				for ( var i = 0, len = slplVids.length; i < len; i++ ) {
					if ( slplVids[i].poster ) {
						slplResource++;
					}
					if ( slplVids[i].preload != 'none' ) {
						slplResource++;
					}
				}
				if ( slplResource <= 20 ) {
					slplInterval = slplSetTimer( 1000 / slplMax, slplMax );
				} else {
					slplInterval = slplSetTimer( Math.min( 15 * slplResource + 700, 4200 ) / slplMax, slplMax );
				}
			</script>
			<?php
		}

		// Add inline script
		public function set_inline_script() {
			$options = $this->options;
			?>
			<script id = "safelayout-cute-preloader-inline-script-js" type = "text/javascript">
				var slplPreLoader = { 
					showingLoader : true,
					pageLoaded : false,
					minShowTime : <?php echo esc_html( $options['minimum_time'] ) * 1000; ?>,
					maxShowTime : <?php echo esc_html( $options['maximum_time'] ) * 1000; ?>,
					showCloseButton : <?php echo esc_html( $options['close_button'] ) * 1000; ?>};
				( function() {
					function slplSetLoaderClass( id, name ) {
						var el = document.getElementById( id );
						if ( el.classList ) {
							el.classList.add( name );
						} else {
							el.className += " " + name;
						}
					}
					function slplHideLoader() {
						if ( slplPreLoader.showingLoader ) {
							slplPreLoader.showingLoader = false;
							if ( typeof slplResourceComplete === 'function' && slplResourceComplete() < 95 ) {
								setTimeout( function() {
										slplSetLoaderClass( 'sl-preloader', 'sl-pl-loaded' );
									},
									200
								);
							} else {
								slplSetLoaderClass( 'sl-preloader', 'sl-pl-loaded' );
							}
							setTimeout( function() {
									document.getElementById( 'sl-preloader' ).remove();
								},
								1300
							);
						}
					}
					document.addEventListener( 'readystatechange', function() {
						if ( document.readyState === 'complete' ) {
							slplPreLoader.pageLoaded = true;
							if ( 0 == slplPreLoader.minShowTime ) {
								slplHideLoader();
							}
						}
					});
					function slplSetCloseButton() {
						if ( slplPreLoader.showingLoader ) {
							var cButton = document.getElementById( 'sl-pl-close-button' );
							cButton.style.display = "block";
							cButton.addEventListener( 'click', function( e ) {
								slplHideLoader();
							});
						}
					}
					if( slplPreLoader.showCloseButton > 0 ) {
						setTimeout( function() {
								slplSetCloseButton();
							},
							slplPreLoader.showCloseButton
						);
					}
					if ( slplPreLoader.maxShowTime > 0 ) {
						setTimeout( function() {
								slplHideLoader();
							},
							slplPreLoader.maxShowTime
						);
					}
					function slplSetMinShowTime() {
						if ( slplPreLoader.pageLoaded ) {
							slplHideLoader();
						} else {
							slplPreLoader.minShowTime = 0;
						}
					}
					if ( slplPreLoader.minShowTime > 0 ) {
						setTimeout( function() {
								slplSetMinShowTime();
							},
							slplPreLoader.minShowTime
						);
					}
				} ) ();
			</script>
			<?php
		}

		// Check if preloader must be shown on this page.
		public function check_active_loader() {
			if ( $this->options === null ) {
				$active = false;
				$this->options = $options = safelayout_preloader_get_options();

				if ( is_customize_preview() || $options['enable_preloader'] != 'enable' ||
					( $options['exclude_404'] === 'enable' && is_404() ) ||
					( $options['exclude_search'] === 'enable' && is_search() && ! have_posts() ) ) {
						$this->active_loader = false;
						return;
				}

				$meta = '';
				$id = $this->get_id();
				if ( $id != 0 ) {
					$meta = trim( get_post_meta( $id, 'safelayout_preloader_shortcode', true ) );
				}
				if ( $meta === '' || substr( $meta, 1, 20 ) != 'safelayout_preloader' ) {
					if ( $options['display_on'] === 'full' ||
						( $options['display_on'] === 'home' && is_front_page() ) ||
						( $options['display_on'] === 'posts' && is_single() ) ||
						( $options['display_on'] === 'pages' && is_page() ) ||
						( $options['display_on'] === 'search' && is_search() ) ||
						( $options['display_on'] === 'archive' && is_archive() ) ) {
							$active = true;
					}

					$type = get_post_type();
					if ( $options['display_on'] === 'custom-id' ) {
						if ( $id != 0 ) {
							$active = $this->check_specific_posts( $options['specific_IDs'], $id );
						}
					}

					if ( $options['display_on'] === 'custom-name' ) {
						if ( $type ) {
							$active = $this->check_specific_posts( $options['specific_names'], $type );
						}
					}

					//exclude name
					if ( $options['exclude_names'] === 'enable' && $active ) {
						if ( $type ) {
							$active = ! $this->check_specific_posts( $options['exclude_names_value'], $type );
						}
					}

					//exclude id
					if ( $options['exclude_IDs'] === 'enable' && $active ) {
						if ( $id != 0 ) {
							$active = ! $this->check_specific_posts( $options['exclude_IDs_value'], $id );
						}
					}
				} else {
					$options = wp_parse_args( shortcode_parse_atts( substr( $meta, 22, -1 ) ), get_option( 'safelayout_preloader_special_post' . $id, array() ) );
					$this->options = $options = wp_parse_args( $options, safelayout_preloader_get_default_options() );
					$active = true;
				}

				$this->options['icon'] = ( strpos( $options['code_HTML'], 'cycle' ) === false ) ? $options['icon'] : 'cycle';
				$this->options['bar_shape'] = ( strpos( $options['code_HTML'], 'anim-border-stripe-bar' ) === false ) ? $options['bar_shape'] : 'anim-border-stripe-bar';

				if ( $options['show_per_session'] && $active ) {
					if ( isset( $_SESSION['safelayout_cute_preloader_shown'] ) ) {
						$active = false;
					} else {
						$_SESSION['safelayout_cute_preloader_shown'] = 'showed';
					}
				}

				if ( ( ! wp_is_mobile() && $options['device'] === 'mobile' ) ||
					( wp_is_mobile() && $options['device'] === 'desktop' ) ) {
						$active = false;
				}

				$this->active_loader = $active;
			}
		}

		// Return curent page, post id
		public function get_id() {
			$id = get_queried_object_id();
			if ( $id === 0 ) {
				global $wp;
				$page = get_page_by_path( $wp->request );
				if ( isset( $page->ID ) ) {
					$id = $page->ID;
				}
			}
			return $id;
		}

		// Return true if id is in specific_posts options
		public function check_specific_posts( $ids, $id ) {
			$ids = explode( ',', $ids );
			$ids = array_map('trim', $ids);
			return in_array( $id, $ids );
		}
	}
	new Safelayout_Preloader_Front();
}
