<?php
/*
Plugin Name: Safelayout Cute Preloader
Plugin URI: https://safelayout.com
Description: Easily add a pure CSS animated preloader to your WordPress website.
Version: 1.8.0
Author: Safelayout
Text Domain: safelayout-cute-preloader
Domain Path: /languages
License: GPLv2
License URI: https://www.gnu.org/licenses/gpl-2.0.html
*/


defined( 'ABSPATH' ) || exit; // Exit if accessed directly.


if ( ! class_exists( 'Safelayout_Preloader' ) && ! class_exists( 'Safelayout_Preloader_Pro' ) ) {

	// Define the constant used in this plugin
	define( 'SAFELAYOUT_PRELOADER_VERSION', '1.8.0');
	define( 'SAFELAYOUT_PRELOADER_NAME', plugin_basename( __FILE__ ) );
	define( 'SAFELAYOUT_PRELOADER_PATH', plugin_dir_path( __FILE__ ) );
	define( 'SAFELAYOUT_PRELOADER_URL', plugin_dir_url( __FILE__ ) );

	// Return default options
	function safelayout_preloader_get_default_options() {
		$default = array(
			'version'					=> SAFELAYOUT_PRELOADER_VERSION,
			'enable_preloader'			=> 'enable',
			'display_on'				=> 'full',
			'specific_IDs'				=> '',
			'specific_names'			=> '',
			'exclude_404'				=> 'enable',
			'exclude_search'			=> '',
			'exclude_IDs'				=> '',
			'exclude_IDs_value'			=> '',
			'exclude_names'				=> '',
			'exclude_names_value'		=> '',
			'show_per_session'			=> 0,
			'device'					=> 'all',
			'inline_style'				=> 0,
			'inline_script'				=> 0,
			'close_button'				=> 5,
			'minimum_time'				=> 0,
			'maximum_time'				=> 0,
			'background_anim'			=> 'linear-right',
			'background_color_type'		=> 'solid',
			'background_color_value'	=> '#101010',
			'background_gradient_value'	=> 25,
			'background_alpha'			=> 90,
			'icon'						=> 'cycle',
			'custom_icon'				=> '',
			'icon_size'					=> 50,
			'icon_color_type'			=> 'solid',
			'icon_color_value'			=> '#4285f4',
			'icon_gradient_value'		=> 5,
			'icon_effect'				=> 0,
			'text'						=> esc_html__( 'Loading ...', 'safelayout-cute-preloader' ),
			'text_anim'					=> 'No',
			'text_size'					=> 14,
			'text_color'				=> '#fff',
			'text_margin_top'			=> 0,
			'brand_url'					=> '',
			'brand_width'				=> 0,
			'brand_height'				=> 0,
			'brand_anim'				=> 'No',
			'brand_position'			=> 'top',
			'brand_margin_top'			=> 0,
			'brand_margin_bottom'		=> 0,
			'bar_shape'					=> 'anim-border-stripe-bar',
			'bar_light'					=> 'enable',
			'bar_position'				=> 'middle_under_text',
			'bar_width'					=> 220,
			'bar_width_unit'			=> 'px',
			'bar_height'				=> 10,
			'bar_border_radius'			=> 10,
			'bar_border_color'			=> '#0f0',
			'bar_margin_top'			=> 0,
			'bar_margin_bottom'			=> 0,
			'bar_margin_left'			=> 0,
			'bar_color_type'			=> 'solid',
			'bar_color_value'			=> '#4285f4',
			'bar_gradient_value'		=> 6,
			'counter'					=> 'enable',
			'counter_text'				=> '%',
			'counter_position'			=> 'center',
			'counter_size'				=> 12,
			'counter_margin_top'		=> 0,
			'counter_margin_bottom'		=> 0,
			'counter_margin_left'		=> 0,
			'counter_color'				=> '#fff',
			'special_meta'				=> '',
			'code_HTML'					=> '<div id="sl-preloader"><div class="sl-pl-bg sl-pl-bg-linear-right"><div></div><div></div><div></div><div></div><div></div><div></div><div></div><div></div><div></div><div></div></div><div id="sl-pl-close-button"><svg class="sl-pl-close-icon" viewbox="0 0 50 50"><path d="M 10,10 L 40,40 M 40,10 L 10,40" /></svg></div><div class="sl-pl-spin-container"><div id="sl-pl-cycle" class="sl-pl-spin"><svg viewBox="0 0 39 39"><defs id="sl-pl-mark"><linearGradient id="sl-pl-cycle-svg-grad01" x1="0" y1="0" x2="1" y2="0"><stop stop-color="#8abcfd" offset="0" /><stop stop-color="#67a5f5" offset="0.44" /><stop stop-color="#5197ec" offset="0.54" /><stop stop-color="#4087dc" offset="1" /></linearGradient></defs><path class="sl-pl-svg-color" stroke="url(#sl-pl-cycle-svg-grad01)" d="M19.5 36.45 C10.1 36.45 2.5 28.85 2.5 19.5 C2.5 10.1 10.1 2.5 19.5 2.5 C28.9 2.45 36.45 10.1 36.45 19.45 C36.45 28.85 28.9 36.4 19.5 36.4" /></svg></div><div class="sl-pl-text">Loading ...</div><div id="sl-pl-bar-middle-container"><div class="sl-pl-bar-container" id="sl-pl-anim-border-stripe-bar-container"><div class="sl-pl-bar-bg"></div><div id="sl-pl-progress"><div class="sl-pl-bar" id="sl-pl-anim-border-stripe-bar"></div><div class="sl-pl-light-move-bar"></div><div id="sl-pl-counter"></div></div></div></div></div></div>',
			'code_CSS'					=> '<noscript><style>#sl-preloader{display: none !important;}</style></noscript><style id="safelayout-cute-preloader-custom-css">.sl-pl-bg{background:rgba(0,0,0,0)!important}.sl-pl-bg-linear-right div{opacity:.9;background:#101010!important}#sl-pl-bar-middle-container{height:16px;margin-top:0!important;margin-bottom:0!important}.sl-pl-bar-container{width:calc(220px - 6px);height:10px;border-radius:10px;border-color:#0f0!important;margin-left:0!important}.sl-pl-bar-bg{background:linear-gradient(0deg,#800000,#f00)}.sl-pl-bar{width:calc(220px - 6px);background:linear-gradient(0deg,#800000,#f00)}#sl-pl-counter{font-size:12px!important;color:#fff!important;margin-top:0;margin-bottom:0;margin-left:0}#sl-pl-counter{text-align:center;top:50%;transform:translateY(-50%);-webkit-transform:translateY(-50%)}.sl-pl-text{font-size:14px!important;color:#fff!important;margin-top:0}.sl-pl-text span{font-size:14px!important;color:#fff!important}.sl-pl-spin{width:50px!important;height:50px!important;transform:scale(1);-webkit-transform:scale(1)}</style>',
		);
		return $default;
	}

	// Return preloader options
	function safelayout_preloader_get_options() {
		$options = wp_parse_args(
			get_option( 'safelayout_preloader_options', array() ),
			safelayout_preloader_get_default_options()
		);
		return $options;
	}

	class Safelayout_Preloader {
		public function __construct() {
			load_plugin_textdomain( 'safelayout-cute-preloader', false, dirname( plugin_basename( __FILE__ ) ) . '/languages' );

			add_filter( 'plugin_action_links_' . SAFELAYOUT_PRELOADER_NAME, array( $this, 'plugin_action_links' ) );

			if ( is_admin() ){
				// Load the admin related functions
				require_once SAFELAYOUT_PRELOADER_PATH . 'inc/class-safelayout-preloader-admin.php';
			} else {
				// Load the front end related functions
				add_action( 'init', array( $this, 'register_session' ) );
				add_action( 'wp', array( $this, 'start_plugin_front' ) );
			}
		}

		// Start session
		public function register_session() {
			if ( ! session_id() ) {
				session_start();
			}
		}

		// Load the front end related functions
		public function start_plugin_front() {
			require_once SAFELAYOUT_PRELOADER_PATH . 'inc/class-safelayout-preloader-front.php';
		}

		// Add settings link on plugin page
		public function plugin_action_links( $links ) {
			$settings_link = array(
				'<a href="' . admin_url( 'options-general.php?page=safelayout-cute-preloader' ) . '">' . esc_html__( 'Settings', 'safelayout-cute-preloader' ) . '</a>',
			);
			$links = array_merge( $links, $settings_link );
			return $links;
		}
	}
	new Safelayout_Preloader();
}